/*******************************************************************************
 * Copyright (c) 2023, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <algorithm>
#include <cstdio>

#include <database.h>

blogi::Database::Database(const char* connstr){
}

blogi::Database::~Database(){
}

int blogi::Database::exec(const SQL &sql,DBResult &res){
    return -1;
}

blogi::SQL::SQL(){
}

blogi::SQL::~SQL()
{
}

blogi::SQL & blogi::SQL::operator<<(const char* sql){
    append(sql,strlen(sql));
    return *this;
}

blogi::SQL & blogi::SQL::operator<<(int sql){
    char buf[512*sizeof(int)];
    snprintf(buf,512,"%d",sql);
    append(buf,strlen(buf));
    return *this;
}

blogi::SQL & blogi::SQL::operator=(const char* sql){
    clear();
    append(sql,strlen(sql));
    return *this;
}

const char* blogi::SQL::escaped(std::vector<char> &buf,const char *text){
    if(!text)
        return nullptr;

    buf.clear();

    size_t tlen=strlen(text);
    for(size_t i = 0; i < tlen; ++i){
        if(text[i]=='\''){
            buf.push_back('\'');
        }
        buf.push_back(text[i]);
    }

    buf.push_back('\0');

    return buf.data();
}


const char * blogi::SQL::c_str() const{
    return _SQL.data();
}

void blogi::SQL::clear(){
    _SQL.clear();
}

bool blogi::SQL::empty() const{
    return _SQL.empty();
}

size_t blogi::SQL::size() const{
    return _SQL.size();
}

void blogi::SQL::append(const char* data, size_t datalen){
    if(!_SQL.empty() && _SQL.back()=='\0')
        _SQL.pop_back();

    std::copy(data,data+datalen,std::back_inserter(_SQL));

    if(_SQL.back()!='\0')
        _SQL.push_back('\0');
}


blogi::DBResult::DBResult(){
    firstRow=nullptr;
}

blogi::DBResult::~DBResult(){
    clear();
}

void blogi::DBResult::clear(){
    Data *curres=firstRow;
    while(curres){
        Data *next=curres->nextData;
        curres->nextData=nullptr;
        delete curres;
        curres = next;
    }
    firstRow=nullptr;
};

blogi::DBResult2 blogi::DBResult::operator[](int value){
    DBResult2 result(this,value);
    return result;
}

blogi::DBResult::Data::Data(int crow,int ccol,const char *value, int len){
    row=crow;
    col=ccol;
    std::copy(value,value+len,
              std::inserter<std::vector<char>>(Column,Column.begin()));
    Column.push_back('\0');
    nextData=nullptr;
}

blogi::DBResult2::DBResult2(DBResult *res,int value){
    result=res;
    row=value;
}

blogi::DBResult2::~DBResult2(){
}

const char *blogi::DBResult2::operator[](int value2){
    for(DBResult::Data *pos=result->firstRow; pos; pos=pos->nextData){
        if( row==pos->row &&  pos->col==value2){
            return pos->Column.data();
        }
    }
    return nullptr;
}




